import click, os
from submit_hpc.job_generator import *

CONTEXT_SETTINGS = dict(help_option_names=['-h','--help'], max_content_width=90)

@click.group(context_settings= CONTEXT_SETTINGS)
@click.version_option(version='0.1')
def job():
    pass

@job.command()
@click.option('-c', '--command', default='', help='Command to execute through torque.', show_default=True)
@click.option('-gpu', '--use_gpu', is_flag=True, help='Specify whether to use GPUs.', show_default=True)
@click.option('-a', '--additions', multiple=True, default=[''], help='Additional commands to add.', show_default=True)
@click.option('-q', '--queue', default='default', help='Queue for torque submission, gpuq also a good one if using GPUs.', show_default=True)
@click.option('-t', '--time', default=1, help='Walltime in hours for job.', show_default=True)
@click.option('-n', '--ngpu', default=0, help='Number of gpus to request.', show_default=True)
@click.option('-ao', '--additional_options', default='', help='Additional options to add for torque run.', type=click.Path(exists=False))
def run_torque_job(command, use_gpu, additions, queue, time, ngpu, additional_options):
    """Run torque job."""
    if isinstance(additions,tuple):
        additions=list(additions)
    elif isinstance(additions,str):
        additions=[additions]
    replace_dict = assemble_replace_dict(command, use_gpu, additions, queue, time, ngpu)
    run_torque_job_(replace_dict, additional_options)

if __name__ == '__main__':
    job()



"""
job_generator.py
=======================
Wraps and runs your commands through torque.
"""

import os

def assemble_replace_dict(command, use_gpu, additions, queue, time, ngpu):
    """Create dictionary to update BASH submission script for torque.

    Parameters
    ----------
    command : type
        Command to executer through torque.
    use_gpu : type
        GPUs needed?
    additions : type
        Additional commands to add (eg. module loads).
    queue : type
        Queue to place job in.
    time : type
        How many hours to run job for.
    ngpu : type
        Number of GPU to use.

    Returns
    -------
    Dict
        Dictionary used to update Torque Script.

    """
    if isinstance(additions,(list,tuple)):
        additions='\n'.join(additions)

    replace_dict = {'COMMAND':"{} {}".format('CUDA_VISIBLE_DEVICES="$gpuNum"' if use_gpu else '',command),
                'GPU_SETUP':"""gpuNum=`cat $PBS_GPUFILE | sed -e 's/.*-gpu//g'`
unset CUDA_VISIBLE_DEVICES
export CUDA_VISIBLE_DEVICES=$gpuNum""" if use_gpu else '',
                'NGPU':'#PBS -l gpus={}'.format(ngpu) if (use_gpu and ngpu) else '',
                'USE_GPU':"#PBS -l feature=gpu" if use_gpu else '',
                'TIME':str(time),'QUEUE':queue,'ADDITIONS':additions}
    return replace_dict

def run_torque_job_(replace_dict, additional_options=""):
    """Run torque job after creating submission script.

    Parameters
    ----------
    replace_dict : type
        Dictionary used to replace information in bash script to run torque job.
    additional_options : type
        Additional options to pass scheduler.

    Returns
    -------
    str
        Custom torque job name.

    """
    txt="""#!/bin/bash -l
#PBS -N run_torque
#PBS -q QUEUE
NGPU
USE_GPU
#PBS -l walltime=TIME:00:00
#PBS -j oe
cd $PBS_O_WORKDIR
GPU_SETUP
ADDITIONS
COMMAND"""
    for k,v in replace_dict.items():
        txt = txt.replace(k,v)
    with open('torque_job.sh','w') as f:
        f.write(txt)
    job=os.popen("mksub {} {}".format('torque_job.sh',additional_options)).read().strip('\n')
    print("Submitted job: {}".format(job))
    return job

def assemble_run_torque(command, use_gpu, additions, queue, time, ngpu, additional_options=""):
    """Runs torque job after passing commands to setup bash file.

    Parameters
    ----------
    command : type
        Command to executer through torque.
    use_gpu : type
        GPUs needed?
    additions : type
        Additional commands to add (eg. module loads).
    queue : type
        Queue to place job in.
    time : type
        How many hours to run job for.
    ngpu : type
        Number of GPU to use.
    additional_options : type
        Additional options to pass to Torque scheduler.

    Returns
    -------
    job
        Custom job name.

    """
    job = run_torque_job_(assemble_replace_dict(command, use_gpu, additions, queue, time, ngpu),additional_options)
    return job



from setuptools import setup
with open('README.md','r', encoding='utf-8') as f:
      long_description = f.read()
setup(name='submit_hpc',
      version='0.1.1',
      description='Collection of growing job submission scripts, not to replace workflow specifications.',
      url='https://github.com/jlevy44/Submit-HPC',
      author='Joshua Levy',
      author_email='joshualevy44@berkeley.edu',
      license='MIT',
      scripts=[],
      entry_points={
            'console_scripts':['submit-job=submit_hpc.job_runner:job']
      },
      long_description=long_description,
      long_description_content_type='text/markdown',
      packages=['submit_hpc'],
      install_requires=['click'])
